//
//  WordCounterAppDelegate.m
//  WordCounter
//
//  Created by Ken Anderson on 4/29/12.
//  Copyright (c) 2012 University of Colorado, Boulder. All rights reserved.
//

#import "WordCounterAppDelegate.h"

@interface WordCounterAppDelegate ()

typedef struct {
    NSUInteger count;
    NSUInteger uniqueWords;
} CountAndSet;

- (void) countWordsWithGCD;
- (void) countWordsWithoutGCD;
- (void) disableEverything;
- (void) enableEverything;
- (void) processResult:(CountAndSet)result;

@end

@implementation WordCounterAppDelegate

@synthesize window = _window;
@synthesize wordsView = _wordsView;
@synthesize countButton = _countButton;
@synthesize useGCD = _useGCD;
@synthesize countLabel = _countLabel;
@synthesize uniqueLabel = _uniqueLabel;
@synthesize spinner = _spinner;

- (void)applicationDidFinishLaunching:(NSNotification *)aNotification {
}

- (CountAndSet) countWords: (NSString *) words {
    NSScanner *scanner = [NSScanner scannerWithString:words];
    NSCharacterSet *whiteSpace = [NSCharacterSet whitespaceAndNewlineCharacterSet];
    NSUInteger count = 0;
    NSCountedSet *wordSet = [NSCountedSet set];
    
    NSString *word;
    while ([scanner scanUpToCharactersFromSet:whiteSpace intoString:&word]) {
        [wordSet addObject:[word lowercaseString]];
        count++;
    }
    
    CountAndSet result = { count, wordSet.count };
    
    sleep(5);
    
    return result;
}

- (void) countWordsWithoutGCD {
    
    CountAndSet result = [self countWords:self.wordsView.string];
    
    [self processResult:result];

}

- (void) countWordsWithGCD {
    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^(void) {
        CountAndSet result = [self countWords:self.wordsView.string];
        dispatch_async(dispatch_get_main_queue(), ^(void) {
            [self processResult:result]; 
        });
    });
}


- (IBAction)count:(id)sender {
    
    [self disableEverything];
    
    if (self.useGCD.state == NO) {
        [self countWordsWithoutGCD];
    } else {
        [self countWordsWithGCD];
    }
    
}

- (void) processResult:(CountAndSet)result {
    
    self.countLabel.stringValue = [NSString stringWithFormat:@"Word Count: %u", result.count];
    
    self.uniqueLabel.stringValue = [NSString stringWithFormat:@"Unique Words: %u", result.uniqueWords];
    
    [self enableEverything];    
}

- (void)disableEverything {
    self.wordsView.editable = NO;
    self.countButton.enabled = NO;
    self.useGCD.enabled = NO;
    self.spinner.hidden = NO;
    [self.spinner startAnimation: self];
}

- (void)enableEverything {
    self.wordsView.editable = YES;
    self.countButton.enabled = YES;
    self.useGCD.enabled = YES;
    [self.spinner stopAnimation: self];
    self.spinner.hidden = YES;
}


@end
